@@ -1,7 +1,10 @@
+from __future__ import print_function
+
 import os
 import sys
 import time
 import signal
+from pprint import pprint
 
 import click
 import yaml
@@ -15,14 +18,7 @@ class DaemonApplication(object):
     config_suffix = "yml"
     default_appname = None
 
-    stop_signal = None          # config item: stop-signal
-    default_pidfile = None      # config item: pidfile
-    default_workspace = None    # config item: workspace
-    default_config = {
-        "stop-signal": signal.SIGINT,
-        "daemon": True,
-        "workspace": os.getcwd(),
-    }
+    # default_config = {}
 
     def get_default_config_filepaths(self, appname, name=None, suffix=None):
         name = name or self.config_name
@@ -65,14 +61,16 @@ class DaemonApplication(object):
         return the_config_file_path
 
     def get_default_config(self):
-        config = {}
+        config = {
+            "pidfile": "app.pid",
+            "stop-signal": signal.SIGINT,
+            "daemon": True,
+            "workspace": os.getcwd(),
+            "loglevel": "INFO",
+            "logfile": "app.log",
+            "logfmt": "default",
+        }
         config.update(getattr(self, "default_config", {}))
-        if not self.default_pidfile is None:
-            config["pidfile"] = self.default_pidfile
-        if not self.default_workspace is None:
-            config["workspace"] = self.default_workspace
-        if not self.stop_signal is None:
-            config["stop-signal"] = self.stop_signal
         return config
 
     def load_config_from_config_file(self, config_file):
@@ -92,27 +90,34 @@ class DaemonApplication(object):
         if config.get("pidfile", None) is None:
             config["pidfile"] = self.appname + ".pid"
 
+    def load_config(self, config, **kwargs):
+        self.appname = self.get_appname()
+        self.config_file_path = self.get_config_file_path(config, self.appname)
+        if self.config_file_path:
+            print("Start application with config file: {}".format(self.config_file_path), file=sys.stderr)
+        else:
+            print("Start application without config file.", file=sys.stderr)
+        self.config = self.get_default_config()
+        self.config.update(self.load_config_from_config_file(self.config_file_path))
+        for key, value in kwargs.items():
+            self.update_config_item(self.config, key.replace("_", "-"), value)
+        self.config["config-file-path"] = self.config_file_path
+        self.fix_config_items(self.config)
+
     def get_main_options(self):
         option_pidfile = click.option("--pidfile", help="pidfile file path.")
         option_daemon = click.option("--daemon/--no-daemon", is_flag=True, default=None, help="Run application in background or in foreground.")
         option_workspace = click.option("--workspace", help="Set running folder")
         option_config = click.option("-c", "--config", help="Config file path. Application will search config file if this option is missing. Use sub-command show-config-fileapaths to get the searching tactics.")
-        return [option_config, option_daemon, option_workspace, option_pidfile]
+        option_loglevel = click.option("--loglevel")
+        option_logfile = click.option("--logfile")
+        option_logfmt = click.option("--logfmt")
+        return [option_config, option_daemon, option_workspace, option_pidfile, option_loglevel, option_logfile, option_logfmt]
 
     def get_controller(self):
         main_options = self.get_main_options()
         def _main(config, **kwargs):
-            self.appname = self.get_appname()
-            self.config_file_path = self.get_config_file_path(config, self.appname)
-            if self.config_file_path:
-                print("Start application with config file: {}".format(self.config_file_path), file=sys.stderr)
-            else:
-                print("Start application without config file.", file=sys.stderr)
-            self.config = self.get_default_config()
-            self.config.update(self.load_config_from_config_file(self.config_file_path))
-            for key, value in kwargs.items():
-                self.update_config_item(self.config, key.replace("_", "-"), value)
-            self.fix_config_items(self.config)
+            self.load_config(config, **kwargs)
         main = _main
         for option in main_options:
             main = option(main)
@@ -158,4 +163,10 @@ class DaemonApplication(object):
             for filepath in config_filepaths:
                 print("    ", filepath)
 
+        @main.command(name="show-configs")
+        def show_configs():
+            """Print out the final config items.
+            """
+            pprint(self.config)
+
         return main
