@@ -99,8 +99,10 @@ class TestDaemonApplication(unittest.TestCase):
         time.sleep(5) # child process may not start yet
         pid = load_pid(pidfile)
 
-        print(pid, type(pid), p.pid, type(p.pid), os.getpid(), type(os.getpid()))
-        assert pid == p.pid
+        if os.name == "nt":
+            assert pid == p.pid # in os nt, daemon flag not working, so the process is not forked, so the the pids are the same.
+        else:
+            assert pid != p.pid # in other os, daemon flag is working, so the process is forked, so the pids are different.
         assert pid != os.getpid()
         assert is_running(pid)
 
